import unittest
import sqlite3
import numpy as np
import pandas as pd
import navipy.database as database
# from navipy.processing.tools import is_numeric_array
import pkg_resources
import tempfile
from navipy.database import DataBaseLoad, DataBaseSave, DataBase


class TestCase(unittest.TestCase):
    def setUp(self):
        self.mydb_filename = pkg_resources.resource_filename(
            'navipy', 'resources/database.db')
        self.mydb = database.DataBaseLoad(self.mydb_filename)

    def test_DataBase_init_(self):
        # filename must end with .db
        with self.assertRaises(NameError):
            DataBase('test')
        # filename must be string
        for n in [2, 5.0, None, np.nan]:
            with self.assertRaises(TypeError):
                DataBase(n)

        # only works if testdb was created before e.g. with DataBaseSave
        with self.assertRaises(NameError):
            DataBase('test')

    def test_DataBase_init_channel(self):
        # filename must end with .db
        for n in [3, 8.7, None, np.nan]:
            with self.assertRaises(TypeError):
                DataBase(self.mydb_filename, n)
        # with self.assertRaises(ValueError):
        #    db=DataBase(self.mydb_filename,[3,4])
        with self.assertRaises(ValueError):
            DataBase(self.mydb_filename, [None, 2])
        # with self.assertRaises(ValueError):
        #    db=DataBase(self.mydb_filename,[4.5,'f'])
        # filename must be string
        for n in [2, 5.0, None, np.nan]:
            with self.assertRaises(TypeError):
                DataBase(n)

        # not working, because DataBase does not create a new database
        with self.assertRaises(NameError):
            DataBase('test', ['R', 'B'])

    def test_table_exist(self):
        # self.assertFalse(self.mydb.table_exist('testblubb'))
        for n in [2, 7.0, None, np.nan]:
            with self.assertRaises(TypeError):
                self.assertFalse(self.mydb.table_exist(n))
        # self.assertFalse(self.mydb.table_exist(self.mydb_filename))

    def test_check_data_validity(self):
        for n in [7.0, None, np.nan]:
            with self.assertRaises(TypeError):
                self.mydb.check_data_validity(n)
        for n in [-1, 0]:
            with self.assertRaises(ValueError):
                self.mydb.check_data_validity(n)
        assert self.mydb.check_data_validity(1)

    def get_posid_test(self):
        conn = sqlite3.connect(self.mydb_filename)
        c = conn.cursor()
        c.execute(""" SELECT * FROM position_orientation WHERE (rowid=1) """)
        rows = c.fetchall()[0]
        # working case
        posorient = pd.Series(index=['x', 'y', 'z',
                                     'alpha_0', 'alpha_1', 'alpha_2'])
        posorient.x = rows[5]
        posorient.y = rows[6]
        posorient.z = rows[1]
        posorient.alpha_0 = rows[3]
        posorient.alpha_1 = rows[2]
        posorient.alpha_2 = rows[4]
        posid = self.mydb.get_posid(posorient)
        assert posid == 1

        # incorrect case missing column
        posorient2 = pd.Series(index=['y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(Exception):
            posid = self.mydb.get_posid(posorient2)

        # incorrect case None
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.x = None
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(ValueError):
            posid = self.mydb.get_posid(posorient2)

        # incorrect case nan
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.x = np.nan
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(ValueError):
            posid = self.mydb.get_posid(posorient2)

        # incorrect case no pandas series but dict
        posorient2 = {}
        posorient2['x'] = posorient.x
        posorient2['y'] = posorient.y
        posorient2['z'] = posorient.z
        posorient2['alpha_0'] = posorient.alpha_0
        posorient2['alpha_1'] = posorient.alpha_1
        posorient2['alpha_2'] = posorient.alpha_2
        with self.assertRaises(TypeError):
            self.mydb.get_posid(posorient2)

        # not working case empty
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])

        with self.assertRaises(Exception):
            self.mydb.get_posid(posorient2)

    def test_DataBaseLoad_init_(self):
        # filename must end with .db
        with self.assertRaises(NameError):
            DataBaseLoad('test')
        # filename must be string
        for n in [2, 5.0, None, np.nan]:
            with self.assertRaises(TypeError):
                DataBaseLoad(n)

        # only working if temporary db was created already
        # e.g. with DataBaseSave
        with self.assertRaises(NameError):
            DataBaseLoad('test')

    def test_read_posorient(self):
        conn = sqlite3.connect(self.mydb_filename)
        c = conn.cursor()
        c.execute(""" SELECT * FROM position_orientation WHERE (rowid=1) """)
        rows = c.fetchall()[0]
        # working case
        posorient = pd.Series(index=['x', 'y', 'z',
                                     'alpha_0', 'alpha_1', 'alpha_2'])
        posorient.x = rows[5]
        posorient.y = rows[6]
        posorient.z = rows[1]
        posorient.alpha_0 = rows[3]
        posorient.alpha_1 = rows[2]
        posorient.alpha_2 = rows[4]
        posid = self.mydb.read_posorient(posorient=posorient)
        assert posid.x == posorient.x

        # incorrect case missing column
        posorient2 = pd.Series(index=['y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(ValueError):
            self.mydb.read_posorient(posorient=posorient2)

        # incorrect case None
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.x = None
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(ValueError):
            self.mydb.read_posorient(posorient=posorient2)

        # incorrect case nan
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.x = np.nan
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(ValueError):
            self.mydb.read_posorient(posorient=posorient2)

        # incorrect case no pandas series but dict
        posorient2 = {}
        posorient2['x'] = posorient.x
        posorient2['y'] = posorient.y
        posorient2['z'] = posorient.z
        posorient2['alpha_0'] = posorient.alpha_0
        posorient2['alpha_1'] = posorient.alpha_1
        posorient2['alpha_2'] = posorient.alpha_2
        with self.assertRaises(TypeError):
            self.mydb.read_posorient(posorient=posorient2)

        # not working case empty
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])

        with self.assertRaises(Exception):
            self.mydb.read_posorient(posorient=posorient2)

    def test_read_posid_id(self):
        conn = sqlite3.connect(self.mydb_filename)
        c = conn.cursor()
        c.execute(""" SELECT * FROM position_orientation WHERE (rowid=1) """)
        rows = c.fetchall()[0]
        # working case
        posorient = pd.Series(index=['id', 'x', 'y', 'z',
                                     'alpha_0', 'alpha_1', 'alpha_2'])
        posorient.id = rows[0]
        posorient.x = rows[5]
        posorient.y = rows[6]
        posorient.z = rows[1]
        posorient.alpha_0 = rows[3]
        posorient.alpha_1 = rows[2]
        posorient.alpha_2 = rows[4]
        for rowid in [0, -2]:
            with self.assertRaises(ValueError):
                # print("rowid",rowid)
                self.mydb.read_posorient(rowid=rowid)
        with self.assertRaises(TypeError):
            self.mydb.read_posorient(rowid='T')
        with self.assertRaises(Exception):
            self.mydb.read_posorient(rowid=None)
        with self.assertRaises(TypeError):
            self.mydb.read_posorient(rowid=np.nan)
        with self.assertRaises(TypeError):
            self.mydb.read_posorient(rowid=4.5)

        for rowid in [1]:
            posoriend2 = self.mydb.read_posorient(rowid=rowid)
            assert posoriend2.id == posorient.id
            assert posoriend2.x == posorient.x
            assert posoriend2.y == posorient.y
            assert posoriend2.z == posorient.z
            assert posoriend2.alpha_0 == posorient.alpha_0
            assert posoriend2.alpha_1 == posorient.alpha_1
            assert posoriend2.alpha_2 == posorient.alpha_2

    def test_scene_id(self):
        for rowid in [0, -2]:
            with self.assertRaises(ValueError):
                # print("rowid",rowid)
                self.mydb.scene(rowid=rowid)
        with self.assertRaises(TypeError):
            self.mydb.scene(rowid='T')
        with self.assertRaises(TypeError):
            self.mydb.scene(rowid=None)
        with self.assertRaises(TypeError):
            self.mydb.scene(rowid=np.nan)
        with self.assertRaises(TypeError):
            self.mydb.scene(rowid=4.5)

        for rowid in [1, 2, 3, 4, 5]:
            image = self.mydb.scene(rowid=rowid)
            # image=np.array(image)
            self.assertIsNotNone(image)
            self.assertFalse(sum(image.shape) == 0)
            self.assertTrue(len(image.shape) == 4)
            self.assertFalse(np.any(np.isnan(image)))
            self.assertTrue(image.shape[3] == 1)
            self.assertTrue(image.shape[2] == 4)
            self.assertTrue(image.shape[0] > 0)
            self.assertTrue(image.shape[1] > 0)

    def test_scene_posorient(self):
        conn = sqlite3.connect(self.mydb_filename)
        c = conn.cursor()
        c.execute(""" SELECT * FROM position_orientation WHERE (rowid=1) """)
        rows = c.fetchall()[0]
        # working case
        posorient = pd.Series(index=['x', 'y', 'z',
                                     'alpha_0', 'alpha_1', 'alpha_2'])
        posorient.x = rows[5]
        posorient.y = rows[6]
        posorient.z = rows[1]
        posorient.alpha_0 = rows[3]
        posorient.alpha_1 = rows[2]
        posorient.alpha_2 = rows[4]
        image = self.mydb.scene(posorient=posorient)
        self.assertIsNotNone(image)
        self.assertFalse(sum(image.shape) == 0)
        # print("shape",image.shape)
        self.assertTrue(len(image.shape) == 4)
        self.assertTrue(image.shape[3] == 1)

        # incorrect case missing column
        posorient2 = pd.Series(index=['y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(Exception):
            image = self.mydb.scene(posorient=posorient2)

        # incorrect case None
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.x = None
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(ValueError):
            image = self.mydb.scene(posorient=posorient2)

        # incorrect case nan
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])
        posorient2.x = np.nan
        posorient2.y = posorient.y
        posorient2.z = posorient.z
        posorient2.alpha_0 = posorient.alpha_0
        posorient2.alpha_1 = posorient.alpha_1
        posorient2.alpha_2 = posorient.alpha_2
        with self.assertRaises(ValueError):
            image = self.mydb.scene(posorient=posorient2)

        # incorrect case no pandas series but dict
        posorient2 = {}
        posorient2['x'] = posorient.x
        posorient2['y'] = posorient.y
        posorient2['z'] = posorient.z
        posorient2['alpha_0'] = posorient.alpha_0
        posorient2['alpha_1'] = posorient.alpha_1
        posorient2['alpha_2'] = posorient.alpha_2
        with self.assertRaises(TypeError):
            image = self.mydb.scene(posorient=posorient2)

        # not working case empty
        posorient2 = pd.Series(index=['x', 'y', 'z',
                                      'alpha_0', 'alpha_1', 'alpha_2'])

        with self.assertRaises(Exception):
            image = self.mydb.scene(posorient=posorient2)

    def test_denormalise_image(self):
        image = self.mydb.scene(rowid=1)
        image = np.squeeze(image)
        cminmaxrange = pd.Series(index=['R_min', 'R_max', 'R_range',
                                        'G_min', 'G_max', 'G_range',
                                        'B_min', 'B_max', 'B_range',
                                        'D_min', 'D_max', 'D_range'])
        cminmaxrange.R_min = 0
        cminmaxrange.R_max = 1
        cminmaxrange.R_range = 1
        cminmaxrange.G_min = 0
        cminmaxrange.G_max = 1
        cminmaxrange.G_range = 1
        cminmaxrange.B_min = 0
        cminmaxrange.B_max = 1
        cminmaxrange.B_range = 1
        cminmaxrange.D_min = 0
        cminmaxrange.D_max = 1
        cminmaxrange.D_range = 1
        imagecorrect = (image.copy()*500).astype(int)
        self.mydb.denormalise_image(imagecorrect, cminmaxrange)

        # not working
        image2 = image.copy().tolist()
        with self.assertRaises(TypeError):
            self.mydb.denormalise_image(image2, cminmaxrange)

        image2 = image[:, :, 0].copy()
        with self.assertRaises(Exception):
            self.mydb.denormalise_image(image2, cminmaxrange)

        image2 = image.copy()
        image2[23, 34, 0] = np.nan
        with self.assertRaises(ValueError):
            self.mydb.denormalise_image(image2, cminmaxrange)

        # image2 = image
        # image2[23,34,0] = 'addsf'
        # with self.assertRaises(ValueError):
        #    denormed=self.mydb.denormalise_image(image2,cminmaxrange)

        image2 = image[np.newaxis, :, :, :]
        with self.assertRaises(Exception):
            self.mydb.denormalise_image(image2, cminmaxrange)

        cminmaxrange2 = pd.Series(index=['R_min', 'R_max', 'R_range',
                                         'B_min', 'B_max', 'B_range',
                                         'D_min', 'D_max', 'D_range'])
        cminmaxrange2.R_min = 0
        cminmaxrange2.R_max = 1
        cminmaxrange2.R_range = 1
        cminmaxrange2.B_min = 0
        cminmaxrange2.B_max = 1
        cminmaxrange2.B_range = 1
        cminmaxrange2.D_min = 0
        cminmaxrange2.D_max = 1
        cminmaxrange2.D_range = 1
        imagecorrect = (image.copy()*500).astype(int)
        with self.assertRaises(ValueError):
            self.mydb.denormalise_image(imagecorrect, cminmaxrange2)

        cminmaxrange3 = pd.Series(index=['R_min', 'R_max', 'R_range',
                                         'G_min', 'G_max', 'G_range',
                                         'B_min', 'B_max', 'B_range',
                                         'D_min', 'D_max', 'D_range'])
        # cminmaxrange3.R_min = []
        # cminmaxrange3.R_max = []
        # cminmaxrange3.R_range = []
        # cminmaxrange3.G_min = []
        # cminmaxrange3.G_max = []
        # cminmaxrange3.G_range = []
        # cminmaxrange3.B_min = []
        # cminmaxrange3.B_max = []
        # cminmaxrange3.B_range = []
        # cminmaxrange3.D_min = []
        # cminmaxrange3.D_max = []
        # cminmaxrange3.D_range = []

        with self.assertRaises(ValueError):
            self.mydb.denormalise_image(imagecorrect, cminmaxrange3)

        cminmaxrange3 = {}
        cminmaxrange3['R_min'] = 0
        cminmaxrange3['R_max'] = 1
        cminmaxrange3['R_range'] = 1
        cminmaxrange3['G_min'] = 0
        cminmaxrange3['G_max'] = 1
        cminmaxrange3['G_range'] = 1
        cminmaxrange3['B_min'] = 0
        cminmaxrange3['B_max'] = 1
        cminmaxrange3['B_range'] = 1
        cminmaxrange3['D_min'] = 0
        cminmaxrange3['D_max'] = 1
        cminmaxrange3['D_range'] = 1
        with self.assertRaises(TypeError):
            self.mydb.denormalise_image(imagecorrect, cminmaxrange3)

        cminmaxrange.R_min = np.nan
        with self.assertRaises(ValueError):
            self.mydb.denormalise_image(imagecorrect, cminmaxrange)

    def test_DataBaseSave(self):
        # should work and creat new database
        with tempfile.TemporaryDirectory() as folder:
            testdb_filename = folder + '/testdatabase.db'
            database.DataBaseSave(testdb_filename)
        # should not work
        with self.assertRaises(Exception):
            database.DataBaseSave(filename=3)

    def test_normalise_image(self):
        image = self.mydb.scene(rowid=1)
        image = np.squeeze(image)
        with tempfile.TemporaryDirectory() as folder:
            testdb_filename = folder + '/testdatabase.db'
            loadDB = DataBaseSave(testdb_filename)
            loadDB.normalise_image(image)

            # not working
            image2 = image.tolist()
            with self.assertRaises(TypeError):
                loadDB.normalise_image(image2)

            image2 = image[:, :, 0].copy()
            with self.assertRaises(Exception):
                loadDB.normalise_image(image2)

            image2 = image.copy()
            image2[23, 34, 0] = np.nan
            with self.assertRaises(ValueError):
                loadDB.normalise_image(image2)

            """image2 = image.copy()
            image2[23,34,0] = 'addsf'
            with self.assertRaises(ValueError):
                denormed=loadDB.normalise_image(image2)"""

            image2 = image[np.newaxis, :, :, :].copy()
            with self.assertRaises(Exception):
                loadDB.normalise_image(image2)

    def test_insert_replace(self):
        params = {}
        params['hight'] = 1.7
        params['age'] = 20
        with tempfile.TemporaryDirectory() as folder:
            testdb_filename = folder + '/testdatabase.db'
            tmpmydb = database.DataBaseSave(testdb_filename)

            for name in [3, 7.5, np.nan, None]:
                with self.assertRaises(TypeError):
                    tmpmydb.insert_replace(name, params)

            for name in [3, 7.5, np.nan, None]:
                with self.assertRaises(TypeError):
                    tmpmydb.insert_replace(name, params)
            with self.assertRaises(sqlite3.OperationalError):
                tmpmydb.insert_replace('test', params)


if __name__ == '__main__':
    unittest.main()
